/*******************************************************************************
 *   Webster II - The Mini Webserver
 *
 *   Application configuration code. This code reads and parses the data in the
 *   file config2.dat found in the root directory of the file system. The
 *   configuration data is then stored in the APP_CONFIG structure for access
 *   by all parts of the software.
 *
 *   Written by: Craig A. Lindley
 *   Version: 1.1
 *   Last Update: 07/30/2008
 ******************************************************************************/
  
#include <basictypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <utils.h>
#include "config.h"
#include "constants.h"
#include "filesystemutils.h"
#include "ucos.h"

// Define the configuration file's name
#define CONFIG_FILE_NAME "config2.dat"
#define MAX_CONFIG_LINES    40
#define MAX_CONFIG_LINE_LEN 256
#define MAX_CONFIG_SIZE (MAX_CONFIG_LINES * MAX_CONFIG_LINE_LEN)

#define CONFIG_OK     0
#define CONFIG_EOF   -1
#define CONFIG_ERROR -2

// Reference to the externally declared AppConfig structure
extern APP_CONFIG AppConfig;

bool configFileRead = FALSE;

#define FILE_BUFFER_SIZE (MAX_CONFIG_SIZE)
static BYTE FILE_BUFFER[FILE_BUFFER_SIZE] __attribute__((aligned(16)));


void showConfig() {
	
	int i = 0;

	iprintf("\r\n\r\nConfiguration Data from config2.dat file\r\n\r\n");
	iprintf("Realm Name: %s\r\n", AppConfig.realmname);
	iprintf("Hostname: %s\r\n", AppConfig.hostname);
	iprintf("Use DHCP: %d\r\n", AppConfig.usedhcp);
	iprintf("Use Auth: %d\r\n", AppConfig.useauth);
	iprintf("Auth User: %s\r\n", AppConfig.authuser);
	iprintf("Auth Pass: %s\r\n", AppConfig.authpass);
	i = 0;
	while (AppConfig.authpages[i][0] != '\0') {
		iprintf("Auth Page%d: %s\r\n", (i + 1), AppConfig.authpages[i]);
		i++;
	}

	iprintf("Use FTP Security: %d\r\n", AppConfig.useftpsecurity);
	iprintf("FTP User: %s\r\n", AppConfig.ftpuser);
	iprintf("FTP Pass: %s\r\n", AppConfig.ftppass);
	
	iprintf("Use NTP: %d\r\n", AppConfig.usentp);
	i = 0;
	while (AppConfig.ntpservers[i] != 0) {
		iprintf("NTP Server%d: ", (i + 1));
		ShowIP(AppConfig.ntpservers[i]);
		iprintf("\r\n");
		i++;
	}
	
	iprintf("Time Zone: %d\r\n", AppConfig.timezone);
	iprintf("Use DST: %d\r\n", AppConfig.use_dst);
	
	iprintf("Static IP Address: ");
	ShowIP(AppConfig.ipaddr);
	iprintf("\r\n");
	iprintf("Static Subnet Mask: ");
	ShowIP(AppConfig.subnetmask);
	iprintf("\r\n");
	iprintf("Static Gateway Address: ");
	ShowIP(AppConfig.gatewayipaddr);
	iprintf("\r\n");
	iprintf("Static DNS Address: ");
	ShowIP(AppConfig.dnsipaddr);
	iprintf("\r\n");

	iprintf("Use Logging: %d\r\n", AppConfig.uselogging);
	iprintf("Logging Hostname: %s\r\n", AppConfig.logginghostname);
}

// Advance pointer past colon and space
BYTE *advPastColonSpace(BYTE *line) {

	BYTE c, *ptr = line;
	c = *ptr++;
	while (c != ':') {
		c = *ptr++;
	}
	return ++ptr;
}

// Advance pointer past comma
BYTE *advPastComma(BYTE *line) {

	BYTE c, *ptr = line;
	c = *ptr++;

	while ((c != ',') && (c != '\0')) {
		c = *ptr++;
	}
	return (c == ',') ? ptr : --ptr;
}

// Get line item from line buffer
void getLineItem(BYTE *inPtr, BYTE *outPtr) {
	
	char c;
	
	*outPtr = '\0';
	
	c = *inPtr++;
	while ((c != ',') && (c != '\0')) {
		*outPtr++ = c;
		c = *inPtr++;
	}
	*outPtr = '\0';
}

// Parse an ASCII IP address into binary
unsigned long parseIP(BYTE *ipStr) {

	return AsciiToIp((char *) ipStr);
}

// Read in the configuration data and return a line each call
int readLine(BYTE *buffer) {

	static BYTE *ptr;
	BYTE c;

	// Has the configuration file already been read in ?
	if (!configFileRead) {
		// No, so attempt to fully read it in now
		if (ReadFile(FILE_BUFFER, CONFIG_FILE_NAME, FILE_BUFFER_SIZE) == 0) {
			// Error reading config file
			return CONFIG_ERROR;
		}
		// Indicate file has been read into memory
		configFileRead = TRUE;

		// Initialize pointer to the data in memory
		ptr = FILE_BUFFER;
	}
	// Return a line from the configuration file
	c = *ptr++;
	if ((c == '\r') || (c == '\0')) {
		return CONFIG_EOF;
	}

	while (c != '\r') {
		*buffer++ = c;
		c = *ptr++;
	}
	*buffer = '\0';
	ptr++;

	return CONFIG_OK;
}

// Read in the configuration file and set the server
// to this configuration. Filesystem must be initialized
// before this call is made.
// Returns TRUE if successful; FALSE otherwise
bool readApplicationConfiguration() {

	BYTE line[MAX_CONFIG_LINE_LEN], buffer[MAX_PAGE_NAME_LENGTH], *ptr;
	int rc, i;

	configFileRead = FALSE;

	// Clear the AppConfig structure to begin with
	memset((void *) &AppConfig, '\0', sizeof(APP_CONFIG));
	
	// Install defaults
	strcpy(AppConfig.realmname, "Netburner Realm");
	strcpy(AppConfig.hostname, "Netburner Host");
	AppConfig.usedhcp = TRUE;
	AppConfig.useauth = FALSE;
	AppConfig.useftpsecurity = FALSE;
	AppConfig.usentp = TRUE;
	AppConfig.ntpservers[0] = AsciiToIp("207.200.81.113");
	AppConfig.uselogging = FALSE;

	while (1) {
		// Read a line from the configuration file
		rc = readLine(line);
		if (rc == CONFIG_ERROR) {
			return FALSE;
		}
		if (rc == CONFIG_EOF) {
			break;
		}

	    if (strncmp((char *) line, "//", 2) == 0) {
			continue;

	    }	else if (strncmp((char *) line, "Realm N", 7) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.realmname, (char *) ptr);

	    }	else if (strncmp((char *) line, "Host", 4) == 0) {
	    	ptr = advPastColonSpace(line);
			strcpy(AppConfig.hostname, (char *) ptr);

	    }	else if (strncmp((char *) line, "Use DHCP", 8) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.usedhcp = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "Use A", 5) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.useauth = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "User", 4) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.authuser, (char *) ptr);

	    }	else if (strncmp((char *) line, "Pass", 4) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.authpass, (char *) ptr);

	    }	else if (strncmp((char *) line, "Pages Req", 9) == 0) {
			ptr = advPastColonSpace(line);
			if (*ptr != '\0') {
				i = 0;
				while (*ptr != '\0') {
					getLineItem(ptr, buffer);
					strcpy(&AppConfig.authpages[i++][0], (char *) buffer);
					ptr = advPastComma(ptr);
				}
			}

	    }	else if (strncmp((char *) line, "Use FTP", 7) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.useftpsecurity = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "FTPUser", 7) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.ftpuser, (char *) ptr);

	    }	else if (strncmp((char *) line, "FTPPass", 7) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.ftppass, (char *) ptr);

	    }	else if (strncmp((char *) line, "Use NTP", 7) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.usentp = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "NTP Servers", 11) == 0) {
			ptr = advPastColonSpace(line);
			if (*ptr != '\0') {
				i = 0;
				while (*ptr != '\0') {
					getLineItem(ptr, buffer);
					AppConfig.ntpservers[i++] = parseIP(buffer);
					ptr = advPastComma(ptr);
				}
			}

	    }	else if (strncmp((char *) line, "Time Zone", 9) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.timezone = atoi((char *) ptr);

	    }	else if (strncmp((char *) line, "Use DST", 7) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.use_dst = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "Static I", 8) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.ipaddr = parseIP(line);

	    }	else if (strncmp((char *) line, "Static S", 8) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.subnetmask = parseIP(ptr);

	    }	else if (strncmp((char *) line, "Static G", 8) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.gatewayipaddr = parseIP(ptr);

	    }	else if (strncmp((char *) line, "Static D", 8) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.dnsipaddr = parseIP(ptr);

	    }	else if (strncmp((char *) line, "Use L", 5) == 0) {
			ptr = advPastColonSpace(line);
			AppConfig.uselogging = (*ptr == 'y') ? TRUE : FALSE;

	    }	else if (strncmp((char *) line, "Logging Host", 12) == 0) {
			ptr = advPastColonSpace(line);
			strcpy(AppConfig.logginghostname, (char *) ptr);
		}
	}
    #ifdef _DEBUG
	showConfig();
	#endif
	
	return TRUE;
}
