/*******************************************************************************
 *   Webster II - The Mini Webserver
 *
 *   This program is made up of NetBurner utility code along
 *   with some code I have written.
 *
 *   Written by: Craig A. Lindley
 *   Version: 1.1
 *   Last Update: 07/30/2008
 ******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <netinterface.h>
#include <autoupdate.h>
#include <ethernet.h>
#include <dhcpclient.h>
#include <dns.h>
#include <effs_fat/mmc_mcf.h>
#include <ftpd.h>
#include <smarttrap.h>
#include <syslog.h>
#include "config.h"
#include "http_f.h"
#include "effs_time.h"
#include "filesystemutils.h"
#include "staticnet.h"

extern "C" {
	void UserMain(void* pd);
}

#define EVENLEDSON 0x55
#define ODDLEDSON  0xAA
#define FTP_PRIO (MAIN_PRIO - 2)

// Declare the name of the application
const char* AppName = "Webster2 Webserver";

// Declare the APP_CONFIG structure
APP_CONFIG AppConfig;

// ShowEthernetLinkStatus() - Show the status of the network link
void ShowEthernetLinkStatus() {
	
	iprintf("\r\n\r\n");
	
	if (EtherLink()) {
		iprintf("Link Status: UP, ");
		
		if (EtherSpeed100()) {
			iprintf("100MB, ");
		}	else	{
			iprintf("10MB, ");
		}
		if (EtherDuplex()) {
			iprintf("Full Duplex\r\n");
		}	else	{
			iprintf("Half Duplex\r\n");
		}
	}	else	{
		iprintf("Link Status: DOWN\r\n");
	}
}

// Assign a runtime IP address for use by the server
void AssignIPAddress() {
	
	// Determine which form of address acquisition is to be used
	iprintf((AppConfig.usedhcp) ? "Using DHCP Addressing" : "Using Static Addressing");

	if (AppConfig.usedhcp) {
		// Use DHCP to get an IP address
		// Set hostname in DHCP request
		pDHCPOfferName = AppConfig.hostname;
		
		if (GetDHCPAddress() == DHCP_OK) {
	 		iprintf("DHCP successful\r\n");
	 	}	else	{
	 		iprintf("DHCP failed\r\n");
	 	}
	 }	else	{
	 	// Use static addressing
	 	// Is the configured network settings the same as the requested one ?
	 	if ((EthernetIP     != AppConfig.ipaddr) ||
	 	    (EthernetIpMask != AppConfig.subnetmask) ||
	 	    (EthernetIpGate != AppConfig.gatewayipaddr) ||
	 	    (EthernetDNS    != AppConfig.dnsipaddr)) {
	 		// Network settings to be changed
			// First, change the network settings stored in flash
 			ChangeFlashNetworkSettings(0,
 									   AppConfig.ipaddr,
 									   AppConfig.subnetmask,
 									   AppConfig.gatewayipaddr,
 									   AppConfig.dnsipaddr);
 			// Second, change the runtime network settings
	 		// Get the first interface identifier
	 		int FirstInterface = GetFirstInterface();
 			ChangeRTNetworkSettings(FirstInterface,
 								    AppConfig.ipaddr,
 								    AppConfig.subnetmask,
 								    AppConfig.gatewayipaddr,
 								    AppConfig.dnsipaddr);
		}
	}
	// Show address assignment results	
	iprintf("Assigned IP address: ");
	ShowIP(EthernetIP);
	iprintf("\n");
	
	ShowEthernetLinkStatus();
}

void SetTime() {
	
	// Use NTP to set the system time ?
	if (AppConfig.usentp) {
		if (SetTimeNTP()) {
			SetTimeZone(AppConfig.timezone, AppConfig.use_dst);
		}
	}
	// Display the time
	DisplaySystemTime();
}

void StartFTP() {
	
	 // Start FTP server with task priority higher than UserMain()
	 int status = FTPDStart(21, FTP_PRIO);
	 if (status == FTPD_OK) {
	 	iprintf("FTP Server started\r\n");
	 	
	 	if (F_LONGFILENAME == 1) {
	 		iprintf("Long file names supported\r\n\r\n");
	 	}	else	{
	 		iprintf("Short file names supported\r\n\r\n");
	 	}
	 }	else	{
	 	iprintf("** Error: %d. Could not start FTP Server\r\n\r\n", status);
	 }
}

////////////////////////////////////////////////////////////////////////////////
// UserMain() - Main program entry point
//
void UserMain(void* pd) {
	
	InitializeStack();
	OSChangePrio(MAIN_PRIO);
	EnableAutoUpdate();
	
 	#ifndef _DEBUG
    EnableSmartTraps();
    #endif

	iprintf("\r\n===== Starting %s Program =====\r\n\r\n", AppName);
	
	f_enterFS();
	
	// We now must also enter the file system for the FTP and HTTP tasks
	OSChangePrio(FTP_PRIO);
	f_enterFS();
	OSChangePrio(HTTP_PRIO);
	f_enterFS();
	OSChangePrio(MAIN_PRIO);

	// Initialize the SD/MMC external flash drive
	InitExtFlash();
	
	// Read in the config file. rc is TRUE on success
	bool rc = readApplicationConfiguration();
	
	// Assign an IP address to the NetBurner module
	AssignIPAddress();
	
    #ifdef _DEBUG
    InitializeNetworkGDB_and_Wait();
    #endif

	// Assign logging host if one was assigned
	IPADDR loggingHostIP;
	SysLogAddress = (GetHostByName(AppConfig.logginghostname, &loggingHostIP, 0, 1200) == DNS_OK)
		? loggingHostIP : NULL;
	
	iprintf("Logging Hostname: %s, SysLogAddress: ", AppConfig.logginghostname);
	ShowIP(SysLogAddress);
	iprintf("\r\n");
	
	// Start the FTP server
	// FTP must be started before infinite loop below so the config2.dat file can be fixed via FTP
	StartFTP();

	// Was the config read successfully ?
	if (!rc) {
		// No so blink onboard LEDs forever
	 	while (1) {
	 		putleds(EVENLEDSON);
	 		OSTimeDly(TICKS_PER_SECOND / 2);
	 		putleds(ODDLEDSON);
	 		OSTimeDly(TICKS_PER_SECOND / 2);
	 	}
	 }
	 // Start the HTTP server
	 StartHTTP();

	 // Register our GET processor
	 RegisterWebFuncs();

	 // Set the time
	 SetTime();
	 
	 // Since main will no longer be accessing the file system we can release it
	 f_releaseFS();
	 
	 while (1) {
	 	OSTimeDly(TICKS_PER_SECOND * 60 * 5);
	 }
}


