/*******************************************************************************
 *   Webster II - The Mini Webserver
 *
 *   This file contains the web server code
 *
 *   Written by: Craig A. Lindley
 *   Version: 1.1
 *   Last Update: 07/30/2008
 ******************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <startnet.h>
#include <effs_fat/mmc_mcf.h>
#include <httppass.h>
#include <syslog.h>
#include <time.h>
#include <tcp.h>
#include "config.h"
#include "effs_time.h"

// Reference to the externally declared AppConfig structure
extern APP_CONFIG AppConfig;

#define LED8ON  0x80
#define LEDSOFF 0x00

#define HTTP_BUFFER_SIZE (32 * 1024) // Make a 32KB BUFFER
static char HTTP_buffer[HTTP_BUFFER_SIZE] __attribute__((aligned(16)));

////////////////////////////////////////////////////////////////////////////////
// SendHeader() - Send an HTTP header of appropriate type for the content
// requested.
//
void sendHeader(int sock, char *mime_txt, long len) {
   char buffer[255];
   siprintf(buffer,
            "HTTP/1.0 200 OK\r\nPragma: no-cache\r\nMIME-version: 1.0\r\nContent-Type: %s\r\nContent-length: %ld\r\n\r\n",
            mime_txt, len);
   writestring(sock, buffer);
}

////////////////////////////////////////////////////////////////////////////////
// SendFile() - Send the contents of the open file to the sock
//
void sendFile(int sock, F_FILE *f, long len) {

   long lread = 0;
   while (lread < len) {
   	
      long ltoread = (len - lread);
      long lr;

      if (ltoread > HTTP_BUFFER_SIZE) {
         ltoread = HTTP_BUFFER_SIZE;
      }

      lr = f_read(HTTP_buffer, 1, HTTP_BUFFER_SIZE, f);

      if (lr == 0) {
         return;
      }
      lread += lr;
      writeall(sock, HTTP_buffer, lr);
   }
}

////////////////////////////////////////////////////////////////////////////////
// OutputHeader
//
void outputHeader(int sock, char *pExt, long len) {
	
	char lcExt[10];
	int extLen = strlen(pExt);
	for (int i = 0; i < extLen; i++) {
		lcExt[i] = tolower(pExt[i]);
	}
	lcExt[extLen] = '\0';
	
//	iprintf("ext: %s - lcExt: %s", pExt, lcExt);
	
	if (strstr(lcExt, "txt") != NULL) {
         sendHeader(sock, "text/plain", len);
	}
	else if (strstr(lcExt, "htm") != NULL) {
    	sendHeader(sock, "text/html", len);
	}
	else if (strstr(lcExt, "xml") != NULL) {
    	sendHeader(sock, "text/xml", len);
	}
	else if (strstr(lcExt, "css") != NULL) {
    	sendHeader(sock, "text/css", len);
	}
	else if (strstr(lcExt, "js") != NULL) {
    	sendHeader(sock, "text/javascript", len);
	}
	else if (strstr(lcExt, "gif") != NULL) {
    	sendHeader(sock, "image/gif", len);
	}
	else if (strstr(lcExt, "bmp") != NULL) {
    	sendHeader(sock, "image/wbmp", len);
	}
	else if (strstr(lcExt, "jpg") != NULL) {
    	sendHeader(sock, "image/jpeg", len);
	}
	else if (strstr(lcExt, "jpeg") != NULL) {
    	sendHeader(sock, "image/jpeg", len);
	}
	else if (strstr(lcExt, "png") != NULL) {
    	sendHeader(sock, "image/png", len);
	}
	else if (strstr(lcExt, "mpeg") != NULL) {
    	sendHeader(sock, "video/mpeg", len);
	}
	else if (strstr(lcExt, "zip") != NULL) {
    	sendHeader(sock, "application/x-zip-compressed", len);
	}
	else if (strstr(lcExt, "jar") != NULL) {
    	sendHeader(sock, "application/x-zip-compressed", len);
	}
	else if (strstr(lcExt, "mp3") != NULL) {
    	sendHeader(sock, "audio/mp3", len);
	}
	else {
		sendHeader(sock, "text/plain", len);
	}
}

////////////////////////////////////////////////////////////////////////////////
// ValidateUser - Function to check username and password for authentication.
// Return 1 for success; 0 otherwise
//
int validateUser(char *pPass, char *pUser) {
	
	int user = strcmp(pUser, AppConfig.authuser);
	int pass = strcmp(pPass, AppConfig.authpass);
	
	if ((user == 0) && (pass == 0)) {
		return 1;
	}	else	{
		return 0;
	}
}

////////////////////////////////////////////////////////////////////////////////
// log() - Logs information about the Get request
//
void log(int sock, PSTR url) {
	
	time_t rawTime;
	struct tm *timeInfo;
	char logEntry[120];

	// Do logging if appropriate
	if (AppConfig.uselogging) {
		// Gather data for log entry
		// Get system time in time_t format
		time(&rawTime);
		// Convert to struct tm format
		timeInfo = localtime(&rawTime);

		// Get the requestor's IP address
		IPADDR requestorIP = GetSocketRemoteAddr(sock);
		PBYTE ipb = (PBYTE) &requestorIP;
 		
		// Format complete log entry
		sprintf(logEntry, 
		"[log time: %s - rip: %d.%d.%d.%d - url: %s]\r\n",
		myasctime(timeInfo),
		(int)ipb[0], (int)ipb[1], (int)ipb[2], (int)ipb[3],
		url);

		// Output the log entry
		SysLog("%s", logEntry);
	}
}

////////////////////////////////////////////////////////////////////////////////
// parseURL() - Break the requested URL into its components
//
void parseURL(char *url, char *name_buffer, char *dir_buffer, char *ext_buffer) {
	
	// Parse and store file extension portion of URL
	char *pext = url + strlen(url);
	
	while ((*pext != '.') && (*pext != '/') && (*pext != '\\') && (pext > url)) {
		pext--;
	}
	
	if ((*pext == '.') || (*pext == '\\') || (*pext == '/')) {
		pext++;
	}
	strcpy(ext_buffer, pext);
	
	// Parse and store file name portion of URL
	char *pName = url + strlen(url);
	
	while ((*pName != '/') && (*pName != '\\') && (pName > url)) {
		pName--;
	}
	
	if ((*pName == '\\') || (*pName == '/')) {
		pName++;
	}
	strcpy(name_buffer, pName);
	
	// Store directory portion of URL
	strncpy(dir_buffer + 1, url, (pName - url));
	dir_buffer[0] = '/';
	dir_buffer[(pName - url) + 1] = 0;
}

////////////////////////////////////////////////////////////////////////////////
// checkPageAuthorization() - This function determines whether authorization is
// is required to access the requested page. Returns TRUE if authorization is
// required.
//
bool checkPageAuthorization(char *pageName) {
	int i = 0;
	
	while (AppConfig.authpages[i][0] != '\0') {
		
		if (strcmp((char *) AppConfig.authpages[i], pageName) == 0) {
			// Found a protected page
			return TRUE;
		}
		i++;
	}
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
// MyDoGet() - This function processes the web browser's GET request.
//
int MyDoGet(int sock, PSTR url, PSTR rxBuffer) {
		
	char name_buffer[32];
	char dir_buffer[128];
	char ext_buffer[10];

	// Turn access LED8 on to indicate a GET
	putleds(LED8ON);

	// Log access request if appropriate
	log(sock, url);

	// Parse the passed in URL
	parseURL(url, name_buffer, dir_buffer, ext_buffer);

	// See if authentication is necessary
	if (AppConfig.useauth && checkPageAuthorization(name_buffer)) {
		
		char *pPass;
		char *pUser;
		
		// Is there username/password info in this request ?
		if (!CheckAuthentication(url, &pPass, &pUser)) {
			// No username/password info provided. Ask the user for it now.
			RequestAuthentication(sock, AppConfig.realmname);
	 		// Turn the access LED off
	 		putleds(LEDSOFF);
			return 1;
		}
		// Username/password info provided in request. Is it valid ?
		if (!validateUser(pPass, pUser)) {
			// Not valid so ask again
			RequestAuthentication(sock, AppConfig.realmname);
	 		// Turn the access LED off
	 		putleds(LEDSOFF);
			return 1;
		}
	}
	// Select the drive and change to root directory
	f_chdrive(MMC_DRV_NUM);
	f_chdir("\\");
		
	// Attempt to change to the specified directory	 
	if (f_chdir(dir_buffer) != F_NO_ERROR) {
		SysLog("ERROR - Directory '%s' does not exist\r\n", dir_buffer);
 		// Turn the access LED off
 		putleds(LEDSOFF);
 		
		// Indicate problem with URL
 		NotFoundResponse(sock, url);
		return 1;
	}
 
	// Directory change OK. Was a filename specified in the URL ?
	if (name_buffer[0] != 0) {
		// Yes a file name was specified in the URL, so attempt to open it
		F_FILE *f = f_open(name_buffer, "r");
		if (f != NULL) {
			long len = f_filelength(name_buffer);

	 		// File was found. Output the appropriate header
	 		outputHeader(sock, ext_buffer, len);
	 			
	 		// Now output the file's data
	 		sendFile(sock, f, len);
	 		f_close(f);
	 		
	 		// Turn the access LED off
	 		putleds(LEDSOFF);
	 		return 0;
	 	}	else	{
	 		// File doesn't exist output error message
	 		SysLog("ERROR - File '%s' does not exist\r\n", name_buffer);

	 		// Turn the access LED off
	 		putleds(LEDSOFF);
		
			// Indicate problem with URL
 			NotFoundResponse(sock, url);
	 		return 1;
	 	}
	}	else	{
		// Filename was not specified so try and locate a file to return.
		// Search for a html file in the following order:
		//  1. index.htm or index.html
		//  2. any file ending in .htm
		//  3. any file ending in .html
		F_FIND f;
	 	int rc = f_findfirst("index.ht*", &f);
	 	if (rc != 0) {
	 		rc = f_findfirst("*.htm", &f);
	 		if (rc != 0) {
	 			rc = f_findfirst("*.html", &f);
	 		}
	 	}
 		// Turn the access LED off
 		putleds(LEDSOFF);

	 	if (rc == 0) {
	 		// Found a file to return so redirect
	 		RedirectResponse(sock, f.filename);
	 		return 0;
	 	}	else	{
	 		// No appropriate file found
	 		SysLog("ERROR - No appropriate html file found\r\n");
	 		
	 		// Indicate problem with URL
 			NotFoundResponse(sock, "No appropriate html file found");
	 		return 1;
	 	}
	}
}


////////////////////////////////////////////////////////////////////////////////
// RegisterWebFuncs() - Register new GET handler.
//
void RegisterWebFuncs() {
	
	SetNewGetHandler(MyDoGet);
}


